@@ -6,23 +6,26 @@ use v5.10.1;
 
 use Test::Roo::Role;
 
+use curry;
+
+use Class::Unload;
 use Path::Tiny;
-use Ref::Util qw/ is_arrayref /;
+use Ref::Util qw/ is_arrayref is_hashref /;
 
 use namespace::autoclean;
 
 requires 'run_tests';
 
-our $VERSION = 'v0.1.4';
+our $VERSION = 'v0.2.0';
 
 
 sub _build_data_files {
-    my ( $class, %args ) = @_;
+    my ( $class, $args ) = @_;
 
-    my $match = $args{match} // qr/\.dat$/;
+    my $match = $args->{match} // qr/\.dat$/;
 
     my @paths = map { path($_) }
-      is_arrayref( $args{files} ) ? @{ $args{files} } : ( $args{files} );
+      is_arrayref( $args->{files} ) ? @{ $args->{files} } : ( $args->{files} );
 
     my @files;
 
@@ -34,8 +37,8 @@ sub _build_data_files {
 
             my $iter = $path->iterator(
                 {
-                    recurse         => $args{recurse}         || 0,
-                    follow_symlinks => $args{follow_symlinks} || 0,
+                    recurse         => $args->{recurse}         || 0,
+                    follow_symlinks => $args->{follow_symlinks} || 0,
                 }
             );
 
@@ -66,54 +69,43 @@ sub run_data_tests {
       : @args;
 
     my $filter = $args{filter} // sub { $_[0] };
+    my $parser = $args{parser} // $class->curry::parse_data_file;
 
-    state $counter = 0;
-
-    $counter++;
-    my $package = __PACKAGE__ . "::Sandbox${counter}";
-
-    foreach my $file ( @{ $class->_build_data_files(%args) } ) {
-
-        state $eval = sub { eval $_[0] };    ## no critic (ProhibitStringyEval)
-
-        my $path = $file->absolute;
+    foreach my $file ( @{ $class->_build_data_files( \%args ) } ) {
 
         note "Data: $file";
 
-        if ( my $data = $eval->("package ${package}; do q{${path}};") ) {
+        my $data = $parser->($file);
 
-            if ( is_arrayref($data) ) {
+        if ( is_arrayref($data) ) {
 
-                my @cases = @$data;
-                my $i     = 1;
+            my @cases = @$data;
+            my $i     = 1;
 
-                foreach my $case (@cases) {
+            foreach my $case (@cases) {
 
-                    my $desc = sprintf(
-                        '%s (%u of %u)',
-                        $case->{description} // $file->basename,    #
-                        $i++,                                       #
-                        scalar(@cases)                              #
-                    );
+                my $desc = sprintf(
+                    '%s (%u of %u)',
+                    $case->{description} // $file->basename,    #
+                    $i++,                                       #
+                    scalar(@cases)                              #
+                );
 
-                    $class->run_tests( $desc, $filter->( $case, $file, $i ) );
+                $class->run_tests( $desc, $filter->( $case, $file, $i ) );
 
-                }
-
-            }
-            else {
-
-                my $desc = $data->{description} // $file->basename;
-
-                $class->run_tests( $desc, $filter->( $data, $file ) );
             }
 
         }
+        elsif ( is_hashref($data) ) {
+
+            my $desc = $data->{description} // $file->basename;
+
+            $class->run_tests( $desc, $filter->( $data, $file ) );
+        }
         else {
 
-            die "parse failed on $file: $@" if $@;
-            die "do failed on $file: $!" unless defined $data;
-            die "run failed on $file" unless $data;
+            my $type = ref $data;
+            die "unsupported data type ${type} returned by ${file}";
 
         }
 
@@ -122,6 +114,28 @@ sub run_data_tests {
 }
 
 
+sub parse_data_file {
+    my ( $class, $file ) = @_;
+
+    my $path = $file->absolute;
+
+    state $eval = sub { eval $_[0] };    ## no critic (ProhibitStringyEval)
+    state $counter = 0;
+
+    my $package = __PACKAGE__ . "::Sandbox" . $counter++;
+
+    my $data = $eval->("package ${package}; do q{${path}} or die \$!;");
+
+    die "parse failed on $file: $@" if $@;
+    die "do failed on $file: $!" unless defined $data;
+    die "run failed or no data returned on $file" unless $data;
+
+    Class::Unload->unload($package);
+
+    return $data;
+}
+
+
 1;
 
 __END__
@@ -136,7 +150,7 @@ Test::Roo::DataDriven - simple data-driven tests with Test::Roo
 
 =head1 VERSION
 
-version v0.1.4
+version v0.2.0
 
 =head1 SYNOPSIS
 
@@ -169,10 +183,13 @@ version v0.1.4
 This class extends L<Test::Roo> for data-driven tests that are kept in
 separate files.
 
-This is useful when a test has hundreds of test cases, where it is
-impractical to include all of the cases in a single test script.
+This is useful when a test script has too many test cases, so that it
+is impractical to include all of the cases in a single test script.
 
-This also allows different tests to share the test cases.
+It allows different tests to share the test cases.
+
+It also makes it easier to have testers with very little Perl
+knowledge to write tests.
 
 =head1 METHODS
 
@@ -195,7 +212,7 @@ The files are expected to be executable Perl snippets that return a
 hash reference or an array reference of hash references.  The keys
 should correspond to the attributes of the L<Test::Roo> class.
 
-See L</DATA FILES> below.
+See L</Data Files> below.
 
 =item C<recurse>
 
@@ -218,7 +235,8 @@ C<qr/\.dat$/>.
 =item C<filter>
 
 This is a reference to a subroutine that takes a single test case as a
-hash reference, as well as the data file and case index in that file.
+hash reference, as well as the data file (L<Path::Tiny>) and case
+index in that file.
 
 The subroutine is expected to return a hash reference to a test case.
 
@@ -238,15 +256,39 @@ use
     ...
   );
 
+=item C<parser>
+
+By default, the data files are Perl snippets. If the data files exist
+in a different format, then an alternative parser can be used.
+
+For example, if the data files were in JSON format:
+
+  MyTests->run_data_tests(
+    match  => qr/\.json$/,
+    parser => sub { decode_json( $_[0]->slurp_raw ) },
+  );
+
+Note that the argument is a L<Path::Tiny> object.
+
+See the L</parse_data_file> method.
+
+Added in v0.2.0.
+
 =back
 
-=for readme stop
+=head2 C<parse_data_file>
 
-=head1 DATA FILES
+  my $data = $class->parse_data_file( $file );
 
-The data files are simple Perl scripts that return a hash reference
-(or array reference of hash references) of constructor values for the
-L<Test::Roo> class..
+This is the default parser for the L</Data Files>.
+
+Added in v0.2.0.
+
+=head3 Data Files
+
+Unless the default L</parser> is changed, the data files are simple
+Perl scripts that return a hash reference (or array reference of hash
+references) of constructor values for the L<Test::Roo> class.
 
 For example,
 
@@ -262,7 +304,7 @@ For example,
     },
   };
 
-Notice in the above example, we are using the C<bag> function from
+In the above example, we are using the C<bag> function from
 L<Test::Deep>, so we have to import the module into our test case to
 ensure that it compiles correctly.
 
@@ -309,6 +351,15 @@ The data files can also include scripts to generate test cases:
     generate_cases( page => 2 ),
   ];
 
+Each data file is loaded into a unique namespace. However, there is
+nothing preventing the datafiles from modifying variables in other
+namespaces, or even doing anything else.
+
+If the data file is successfully parsed, then the namespace is
+unloaded.
+
+=for readme stop
+
 =head1 KNOWN ISSUES
 
 =head2 Skipping test cases
@@ -329,7 +380,7 @@ will stop all remaining tests from running.
 =head2 Prerequisite Scanners
 
 Prerequisite scanners used for build tools may not recognise modules
-used in the L</DATA FILES>.  To work around this, use the modules as
+used in the L</Data Files>.  To work around this, use the modules as
 well in the test class or explicitly add them to the distribution's
 metadata.
 
